/* R-U-ON Vista Gadget           */
/* Copyright (c) 2007 R-U-ON LLC */
/* www.r-u-on.com                */

// Consts
var ruonServer = "http://rss.r-u-on.com/";
var url = { "alarms":ruonServer+"rssalarms?vista&id=",
            "agents":ruonServer+"rssagents?vista&id=",
            "tickets":ruonServer+"rsstickets?vista&id=",
            "guid": "https://www.r-u-on.com/resolveGUID?"
};
var alarmParseFields = [ null, "Description", "Time Up" ];

function authenticate(u, p, result) {

    var req = new XMLHttpRequest();
    req.onreadystatechange = accountId;
    req.open("GET", url["guid"]+'email='+encodeURIComponent(u)+"&password="+encodeURIComponent(p), true);
    req.send();
    function accountId() {
        if (req.readyState == 4) {
            if (req.status == 200) {
                try {
                    req.responseXML.loadXML(req.responseText);
                    parseGUID(req.responseXML, result);
                } catch (e) {
                    result(false, "Internal Error: "+e.message);
                }
            } else {
                result(false, "Can't connect to server");
            }
        }
    }
}

function parseGUID(doc, result) {
    if (doc.parseError.errorCode!=0) {
        result(false, "Temporary error. Please try again.");
        return;
    }
    var node = doc.selectSingleNode("//error");
    if (node!=null) {
        result(false, node.text);
    } else {
        node = doc.selectSingleNode("//guid");
        if (node!=null) {
            accountid = node.text;
            result(true, accountid);
        } else {
            result(false, "Unknow result. Please try again later.")
        }
    }
}

function fetchData(uid, mode, group, result) {
    var xml_request = new XMLHttpRequest();
    xml_request.onreadystatechange = function() {xmlResult(xml_request, mode, result);}
    xml_request.open("GET", buildUrl(uid, mode, group));
    xml_request.setRequestHeader("Cache-Control", "no-cache");
    xml_request.send();
}

function buildUrl(uid, mode, group) {
    return url[mode] + uid +(group?"&group":"");
}

function xmlResult(req, m, result) {
    
    if (req.readyState == 4) {
        var info;
        if (req.status == 200) {
            req.responseXML.loadXML(req.responseText);
            info = parseFeed(m, req.responseXML);
        } else {
            if (req.status==500) {
                info = exceptionInfo("Service Unavailable", "The R-U-ON Service is temporarily unavailable.")
            } else {
                if (req.status) {
                    info = exceptionInfo("Service unavailable", "HTTP Status code "+req.status);
                } else {
                    info = exceptionInfo("Service/Network unavailable", "Can't connect to server");
                }
            }
        }
        result(info);
    }    
}
function parseFeed(m, doc) {
    //try {
        var items = doc.getElementsByTagName("item");
        var result = new Object();
        result.ttl = parseInt(doc.getElementsByTagName("ttl").item(0).text);

        // Is Exception
        if (items.length==1) {
            var title = items[0].getElementsByTagName("title").item(0).text;
            if (title=="R-U-ON Feed Exception") {
                result = exceptionInfo(title, items[0].getElementsByTagName("description").item(0).text);
                return result;
            }
        }

        if (m=="alarms") {
            result.type = "alarms";
            result.rows = [];
            for (var i=0;i<items.length;++i) {
                result.rows[i] = parseAlarm(items[i]);
            }
        } else if (m==="agents") {
            result.type = "agents";
            result.rows = [];
            for (var i=0;i<items.length;++i) {
                result.rows[i] = parseAgent(items[i]);
            }
        } else if (m=="tickets") {
            result.type = "tickets";
            result.rows = [];
            for (var i=0;i<items.length;++i) {
                result.rows[i] = parseTicket(items[i]);
            }
                   
        } else {
            result = exceptionInfo("Unknown mode", "mode=\""+m+"\"");
        }
//    } catch(e) {
        //result = exceptionInfo("Parsing Error", e.message);
    //}
    return result;
}
function parseAlarm(node) {
    var alarm = new Array();
    var title = node.getElementsByTagName("title").item(0).text;
    var cdata = node.getElementsByTagName("description").item(0).text;

    var ix = title.lastIndexOf(": ");
    alarm['Agent'] = title.substr(0, ix);
    alarm['Severity'] = title.substr(ix+2, title.length-ix-2);

    var cp = cdata.split("<br>");
    for (var i=0;i<cp.length;++i) {
        if (alarmParseFields[i]==null) {
            var s = cp[i];
            var idx = s.indexOf(": ");
            var k = s.substr(0, idx);
            var v = s.substr(idx+2, s.length - idx - 2);
            alarm[k] = v;
        } else {
            alarm[alarmParseFields[i]] = cp[i];
        }
    }

    alarm['link'] = node.getElementsByTagName("link").item(0).text;
    alarm['guid'] = node.getElementsByTagName("guid").item(0).text;
    alarm['severity'] = alarm['Severity'];

    return alarm;
}
function parseAgent(node) {
    var agent = [];
    var title = node.getElementsByTagName("title").item(0).text;
    var cdata = node.getElementsByTagName("description").item(0).text;

    agent['Agent'] = title;

    var cp = cdata.split("<br>");
    for (var i=0;i<cp.length;++i) {
        var s = cp[i];
        var idx = s.indexOf(": ");
        var k = s.substr(0, idx);
        var v = s.substr(idx+2, s.length - idx - 2);
        agent[k] = v;
    }
    agent['link'] = node.getElementsByTagName("link").item(0).text;
    agent['guid'] = node.getElementsByTagName("guid").item(0).text;
    agent['text'] = cdata;
    agent['severity'] = agent['Severity'];
    return agent;
}
function parseTicket(node) {
    var ticket = [];
    var title = node.getElementsByTagName("title").item(0).text;
    var cdata = node.getElementsByTagName("description").item(0).text;

    var ix = title.indexOf(": ");
    ticket['TRK#'] = title.substr(1, ix-1);
    ticket['Title'] = title.substr(ix+2, title.length-ix-2);


    var cp = cdata.split("<br>");
    for (var i=0;i<cp.length;++i) {
        var s = cp[i];
        var idx = s.indexOf(": ");
        var k = s.substr(0, idx);
        var v = s.substr(idx+2, s.length - idx - 2);
        ticket[k] = v;
    }
    ticket['link'] = node.getElementsByTagName("link").item(0).text;
    ticket['guid'] = node.getElementsByTagName("guid").item(0).text;
    ticket['text'] = cdata;
    ticket["Group"] = ticket["Category"];
    if (ticket["Group"]==null) {
        ticket["Group"] = "None";
    }

    if (ticket["Status"]=="Closed") {
        ticket["severity"] = "Okay";
    } else if (ticket["Status"]=="Resolved") {
        ticket["severity"] = "Resolved";
    } else {
        ticket["severity"] = ticket["Severity"];
    }
    ticket["Description"] = ticket["Title"];
    
    ticket["Opened By Trunc"] = emailName(ticket["Opened By"]);
    
    return ticket;
}

function emailName(email) {
    if (email==null) {
        return null;
    }
    var ix = email.indexOf("@");
    if (ix==-1) {
        return  email;
    }
    return email.substr(0, ix);
}


function exceptionInfo(title, description) {
    var result = new Object();

    result.type = "exception";
    result.title = title;
    result.description = description;
    return result;
}
