/* R-U-ON Vista Gadget           */
/* Copyright (c) 2007 R-U-ON LLC */
/* www.r-u-on.com                */

function Scrollbar(container, panel, site) {
    this.panel = panel;
    this.container = container;
    this.site = site;
    
    this.hookup();
}

Scrollbar.prototype.hookup = function() {
    this.site.style.display = "";
    this.site.style.backgroundColor = "#eeeeee"
    this.site.style.border = "1px solid #ffffff";
    
    this.thumb = document.createElement ("img");
    this.thumb.src = "img/pxl.png";
    this.thumb.style.backgroundColor = "#bbbbbb";
    this.thumb.style.position = "relative";
    this.thumb.style.width = "5px";
    this.thumb.style.top = "0px";
    this.thumb.style.left = "0px";
    this.site.appendChild(this.thumb);

    var self = this;
    this.site.onmousewheel = function() { self.mouseWheel(); };
    this.panel.onmousewheel = function() { self.mouseWheel(); };
    this.site.onmousedown = function() { self.mouseDown(); };
    this.thumb.onmouseup = function() {self.thumbUp();};
    this.thumb.onmousemove = function() {self.thumbMove();};
    document.onkeydown = function() {self.keyDown();};
    this.resize();
}

Scrollbar.prototype.clear= function() {
    this.site.style.display = "none";
    this.panel.style.top = "0px";
    this.site.removeChild(this.thumb);
    this.site.onmousedown = null;
    this.site.onmousewheel = null;
    this.panel.onmousewheel = null;
    document.onkeydown = null;
}

Scrollbar.prototype.resize = function() {
    var h = (this.container.offsetHeight/this.panel.offsetHeight)*this.site.offsetHeight;
    h = Math.max(h, 20);
    h = Math.min(h, this.site.offsetHeight - 20);
    this.thumb.style.height = h+"px";
    this.forceHFIX = true;
    this.travelTo(this.panel.offsetTop);
}

Scrollbar.prototype.mouseWheel = function() {
    this.scrollBy(event.wheelDelta<0?10:-10);
}

Scrollbar.prototype.scrollBy = function(step) {
    this.travelTo(this.panel.offsetTop-step);
}

Scrollbar.prototype.travelTo = function(where) {
    where = Math.min(0, where);
    where = Math.max(where,this.container.offsetHeight-this.panel.offsetHeight);
    this.panel.style.top = where+"px";
    this.thumbPosFromPix();
}

Scrollbar.prototype.thumbPosFromPix = function() {
    var range = this.container.offsetHeight-this.panel.offsetHeight;
    var percent = this.panel.offsetTop/range;
    range = this.site.offsetHeight - this.thumb.offsetHeight-2;
    var top = percent*range;
    
    if (this.forceHFIX) {
        //The thumb just changed size, so we need this work around (MS BUG)
        this.forceHFIX = false;
        this.thumb.style.top = (top-1)+"px";
        var self = this;
        wafunc = function() {
            self.thumb.style.top = top+"px";
        }
        setTimeout("wafunc()", 1);
    } else {
        this.thumb.style.top = top+"px";
    }
}

Scrollbar.prototype.mouseDown = function() {

    var y = event.offsetY;
    var thumbY = parseInt(this.thumb.style.top);
    if (event.srcElement==this.thumb) {
        y+=thumbY;
    }

    if (y<thumbY) {
        this.scrollBy(-this.pageY());
    } else if (y>thumbY+this.thumb.offsetHeight) {
        this.scrollBy(this.pageY());
    } else {
        this.thumb.setCapture();
        this.dragScreenYStart = event.screenY;
        this.dragThumbYStart = parseInt(this.thumb.style.top);
        this.dragging = true;
    }
}

Scrollbar.prototype.thumbUp = function() {
    this.thumb.releaseCapture();
    this.dragging = false;
    event.cancelBubble=true;
}

Scrollbar.prototype.thumbMove = function() {
    if (this.dragging) {
        var offset = event.screenY - this.dragScreenYStart;
        this.placeThumb(this.dragThumbYStart+offset);
        event.cancelBubble=true;
    }
}

Scrollbar.prototype.placeThumb = function(y) {
    var range = this.site.offsetHeight - this.thumb.offsetHeight-2;
    y = Math.max(0, y);
    y = Math.min(y, range);
    this.thumb.style.top = y+"px";

    var percent = y/range;
    range = this.container.offsetHeight-this.panel.offsetHeight;
    this.panel.style.top = percent*range+"px";
}

Scrollbar.prototype.pageY = function() {
    return this.container.offsetHeight-20;
}

Scrollbar.prototype.keyDown = function() {
    var scrl = {
        40:20, 38:-20,
        34:this.pageY(), 33:-this.pageY(),
        32:20,
        36:-999999,
        35:999999
    };
    var y = scrl[event.keyCode];
    if (y) {
        this.scrollBy(y);
    }
}


